@@ -4,7 +4,7 @@
 ##
 ##    class "fv" of function value objects
 ##
-##    $Revision: 1.148 $   $Date: 2017/07/13 01:26:02 $
+##    $Revision: 1.154 $   $Date: 2019/05/24 10:12:16 $
 ##
 ##
 ##    An "fv" object represents one or more related functions
@@ -95,7 +95,7 @@ fv <- function(x, argu="r", ylab=NULL, valu, fmla=NULL,
   attr(x, "alim") <- alim
   attr(x, "labl") <- labl
   attr(x, "desc") <- desc
-  attr(x, "units") <- as.units(unitname)
+  attr(x, "units") <- as.unitname(unitname)
   attr(x, "fname") <- fname
   attr(x, "dotnames") <- NULL
   attr(x, "shade") <- NULL
@@ -280,13 +280,21 @@ fvnames <- function(X, a=".") {
              dn <- fvnames(X, "*")
            return(dn)
          },
-         "*"=,
          ".a"={
            ## all column names other than the function argument
            allvars <- names(X)
            argu <- attr(X, "argu")
            nam <- allvars[allvars != argu]
-           nam <- rev(nam) ## convention
+           return(nam)
+         },
+         "*"={
+           ## Not documented at user level
+           ## All column names other than the function argument
+           ##           IN REVERSE ORDER
+           allvars <- names(X)
+           argu <- attr(X, "argu")
+           nam <- allvars[allvars != argu]
+           nam <- rev(nam) # NB
            return(nam)
          },
          {
@@ -307,10 +315,10 @@ fvnames <- function(X, a=".") {
     return(X)
   }
   if(a == ".a" || a == "*") {
-    warning("Column names unchanged: use names(x) <- value to change them")
+    warning("Nothing changed; use names(X) <- value to change names",
+            call.=FALSE)
     return(X)
   }
-
   ## validate the names
   switch(a,
          ".x"=,
@@ -367,6 +375,10 @@ fvnames <- function(X, a=".") {
   fvnames(x, ".")  <- value[ind.]
   if(length(inds) > 0)
     fvnames(x, ".s") <- value[inds]
+  namemap <- setNames(lapply(value, as.name), nama)
+  formula(x) <- flat.deparse(eval(substitute(substitute(fom, um),
+                                             list(fom=as.formula(formula(x)),
+                                                  um=namemap))))
   return(x)
 }
 
@@ -1348,7 +1360,7 @@ distributecbind <- local({
 
 ratfv <- function(df, numer, denom, ..., ratio=TRUE) {
   ## Determine y
-  if(!missing(df)) {
+  if(!missing(df) && !is.null(df)) {
     y <- fv(df, ...)
     num <- NULL
   } else {
@@ -1407,12 +1419,12 @@ bind.ratfv <- function(x, numerator=NULL, denominator=NULL,
     both <- numerator
     denominator <- attr(both, "denominator")
     usenames <- fvnames(both, ".a")
-    numerator   <- as.data.frame(both)[,usenames]
-    denominator <- as.data.frame(denominator)[,usenames]
+    numerator   <- as.data.frame(both)[,usenames,drop=FALSE]
+    denominator <- as.data.frame(denominator)[,usenames,drop=FALSE]
     ##  labels default to those of ratio object
-    if(is.null(labl)) labl <- attr(both, "labl")
-    if(is.null(desc)) desc <- attr(both, "desc")
-    if(is.null(labl)) labl <- attr(both, "labl")
+    ma <- match(usenames, colnames(both))
+    if(is.null(labl)) labl <- attr(both, "labl")[ma]
+    if(is.null(desc)) desc <- attr(both, "desc")[ma]
   }
   # calculate ratio
   #    The argument 'quotient' is rarely needed 
